@@ -24,8 +24,7 @@ import gtk.glade
 import traceback
 import logging
 
-import libvirt
-
+from virtManager import util
 from virtManager.error import vmmErrorDialog
 from virtManager.asyncjob import vmmAsyncJob
 from virtManager.createmeter import vmmCreateMeter
@@ -301,12 +300,7 @@ class vmmCreatePool(gobject.GObject):
         newconn = None
         try:
             # Open a seperate connection to install on since this is async
-            logging.debug("Threading off connection to create pool.")
-            #newconn = vmmConnection(self.config, self.conn.get_uri(),
-            #                        self.conn.is_read_only())
-            #newconn.open()
-            #newconn.connectThreadEvent.wait()
-            newconn = libvirt.open(self._pool.conn.getURI())
+            newconn = util.dup_conn(self.config, None, self._pool.conn)
             meter = vmmCreateMeter(asyncjob)
             self._pool.conn = newconn
 
@@ -407,20 +401,9 @@ class vmmCreatePool(gobject.GObject):
         mode = gtk.FILE_CHOOSER_ACTION_OPEN
         if foldermode:
             mode = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER
-        fcdialog = gtk.FileChooserDialog(dialog_name, self.topwin, mode,
-                                         (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                          gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT),
-                                         None)
-        fcdialog.set_default_response(gtk.RESPONSE_ACCEPT)
-        if startfolder != None:
-            fcdialog.set_current_folder(startfolder)
 
-        response = fcdialog.run()
-        fcdialog.hide()
-        ret = None
-        if(response == gtk.RESPONSE_ACCEPT):
-            ret = fcdialog.get_filename()
-        fcdialog.destroy()
-        return ret
+        return util.browse_local(self.topwin, dialog_name, dialog_type=mode,
+                                 start_folder=startfolder,
+                                 foldermode=foldermode)
 
 gobject.type_register(vmmCreatePool)
