@@ -24,8 +24,7 @@ import gtk.glade
 import traceback
 import logging
 
-import libvirt
-
+from virtManager import util
 from virtManager.error import vmmErrorDialog
 from virtManager.asyncjob import vmmAsyncJob
 from virtManager.createmeter import vmmCreateMeter
@@ -81,14 +80,18 @@ class vmmCreateVolume(gobject.GObject):
 
 
     def show(self):
+        self.reset_state()
         self.topwin.show()
         self.topwin.present()
-        self.reset_state()
 
     def close(self, ignore1=None, ignore2=None):
         self.topwin.hide()
+        self.set_modal(False)
         return 1
 
+    def set_modal(self, modal):
+        self.topwin.set_modal(bool(modal))
+
     def set_parent_pool(self, pool):
         self.parent_pool = pool
         self.vol_class = Storage.StoragePool.get_volume_for_pool(self.parent_pool.get_type())
@@ -141,8 +144,8 @@ class vmmCreateVolume(gobject.GObject):
             if not self.validate():
                 return
         except Exception, e:
-            self.err.show_err(_("Uncaught error validating input: %s") % str(e),
-                                "".join(traceback.format_exc()))
+            self.show_err(_("Uncaught error validating input: %s") % str(e),
+                            "".join(traceback.format_exc()))
             return
 
         logging.debug("Creating volume with xml:\n%s" %
@@ -160,7 +163,7 @@ class vmmCreateVolume(gobject.GObject):
         progWin.run()
 
         if self.error_msg is not None:
-            self.err.show_err(self.error_msg, self.error_details)
+            self.show_err(self.error_msg, self.error_details)
             self.topwin.set_sensitive(True)
             self.topwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.TOP_LEFT_ARROW))
             return
@@ -173,13 +176,7 @@ class vmmCreateVolume(gobject.GObject):
     def _async_vol_create(self, asyncjob):
         newconn = None
         try:
-            # Open a seperate connection to install on since this is async
-            logging.debug("Threading off connection to create vol.")
-            #newconn = vmmConnection(self.config, self.conn.get_uri(),
-            #                        self.conn.is_read_only())
-            #newconn.open()
-            #newconn.connectThreadEvent.wait()
-            newconn = libvirt.open(self.conn.get_uri())
+            newconn = util.dup_conn(self.config, self.conn)
 
             # Lookup different pool obj
             newpool = newconn.storagePoolLookupByName(self.parent_pool.get_name())
@@ -212,4 +209,8 @@ class vmmCreateVolume(gobject.GObject):
             return self.err.val_err(_("Volume Parameter Error"), str(e))
         return True
 
+    def show_err(self, info, details):
+        async = not self.topwin.get_modal()
+        self.err.show_err(info, details, async=async)
+
 gobject.type_register(vmmCreateVolume)
